@@ -10,11 +10,14 @@ except ImportError:
 class Backend(BaseBackend):
     """Backend using the Django cache framework to store task metadata."""
 
+    capabilities = ["ResultStore"]
+
     def __init__(self, *args, **kwargs):
         super(Backend, self).__init__(*args, **kwargs)
         self._cache = {}
 
     def _cache_key(self, task_id):
+        """Get the cache key for a task by id."""
         return "celery-task-meta-%s" % task_id
 
     def store_result(self, task_id, result, status):
@@ -28,11 +31,11 @@ class Backend(BaseBackend):
 
     def get_status(self, task_id):
         """Get the status of a task."""
-        return self._get_task_meta_for(self, task_id)["status"]
+        return self._get_task_meta_for(task_id)["status"]
 
     def get_result(self, task_id):
         """Get the result of a task."""
-        meta = self._get_task_meta_for(self, task_id)
+        meta = self._get_task_meta_for(task_id)
         if meta["status"] == "FAILURE":
             return self.exception_to_python(meta["result"])
         else:
@@ -43,6 +46,7 @@ class Backend(BaseBackend):
         return self.get_status(task_id) == "DONE"
 
     def _get_task_meta_for(self, task_id):
+        """Get the task metadata for a task by id."""
         if task_id in self._cache:
             return self._cache[task_id]
         meta = cache.get(self._cache_key(task_id))
