@@ -7,13 +7,12 @@ DEFAULT_AMQP_PUBLISHER_ROUTING_KEY = "celery"
 DEFAULT_AMQP_CONSUMER_ROUTING_KEY = "celery"
 DEFAULT_AMQP_CONSUMER_QUEUE = "celery"
 DEFAULT_AMQP_EXCHANGE_TYPE = "direct"
-DEFAULT_DAEMON_CONCURRENCY = 10
-DEFAULT_QUEUE_WAKEUP_AFTER = 0.3
-DEFAULT_EMPTY_MSG_EMIT_EVERY = 5
+DEFAULT_DAEMON_CONCURRENCY = 0 # defaults to cpu count
 DEFAULT_DAEMON_PID_FILE = "celeryd.pid"
 DEFAULT_LOG_FMT = '[%(asctime)s: %(levelname)s/%(processName)s] %(message)s'
 DEFAULT_DAEMON_LOG_LEVEL = "INFO"
 DEFAULT_DAEMON_LOG_FILE = "celeryd.log"
+DEFAULT_AMQP_CONNECTION_TIMEOUT = 4
 
 """
 .. data:: LOG_LEVELS
@@ -61,28 +60,6 @@ DAEMON_LOG_FILE = getattr(settings, "CELERYD_LOG_FILE",
 DAEMON_LOG_LEVEL = LOG_LEVELS[getattr(settings, "CELERYD_DAEMON_LOG_LEVEL",
                                       DEFAULT_DAEMON_LOG_LEVEL).upper()]
 
-"""
-.. data:: QUEUE_WAKEUP_AFTER
-
-    The time (in seconds) the celery worker should sleep when there's
-    no messages left on the queue. After the time is slept, the worker
-    wakes up and checks the queue again.
-
-"""
-QUEUE_WAKEUP_AFTER = getattr(settings, "CELERYD_QUEUE_WAKEUP_AFTER",
-                             DEFAULT_QUEUE_WAKEUP_AFTER)
-
-"""
-.. data:: EMPTY_MSG_EMIT_EVERY
-
-    How often the celery daemon should write a log message saying there are no
-    messages in the queue. If this is ``None`` or ``0``, it will never print
-    this message.
-
-"""
-EMPTY_MSG_EMIT_EVERY = getattr(settings, "CELERYD_EMPTY_MSG_EMIT_EVERY",
-                               DEFAULT_EMPTY_MSG_EMIT_EVERY)
-
 """
 .. data:: DAEMON_PID_FILE
 
@@ -155,6 +132,15 @@ AMQP_CONSUMER_QUEUE = getattr(settings, "CELERY_AMQP_CONSUMER_QUEUE",
                               DEFAULT_AMQP_CONSUMER_QUEUE)
 
 """
+.. data:: AMQP_CONNECTION_TIMEOUT
+
+    The timeout in seconds before we give up establishing a connection
+    to the AMQP server.
+
+"""
+AMQP_CONNECTION_TIMEOUT = getattr(settings, "CELERY_AMQP_CONNECTION_TIMEOUT",
+                                  DEFAULT_AMQP_CONNECTION_TIMEOUT)
+"""
 .. data:: SEND_CELERY_TASK_ERROR_EMAILS
 
     If set to ``True``, errors in tasks will be sent to admins by e-mail.
