@@ -24,13 +24,13 @@ class TaskRegistry(UserDict):
         """Autodiscovers tasks using :func:`celery.discovery.autodiscover`."""
         discovery.autodiscover()
 
-    def register(self, task, task_name=None):
+    def register(self, task, name=None):
         """Register a task in the task registry.
 
         Task can either be a regular function, or a class inheriting
         from :class:`celery.task.Task`.
 
-        :keyword task_name: By default the :attr:`Task.name` attribute on the
+        :keyword name: By default the :attr:`Task.name` attribute on the
             task is used as the name of the task, but you can override it
             using this option.
 
@@ -39,34 +39,35 @@ class TaskRegistry(UserDict):
         """
         is_class = hasattr(task, "run")
 
-        if not task_name:
-            task_name = getattr(task, "name")
+        if not name:
+            name = getattr(task, "name")
 
-        if task_name in self.data:
+        if name in self.data:
             raise self.AlreadyRegistered(
-                    "Task with name %s is already registered." % task_name)
+                    "Task with name %s is already registered." % name)
 
         if is_class:
-            self.data[task_name] = task() # instantiate Task class
+            self.data[name] = task() # instantiate Task class
         else:
-            task.name = task_name
+            task.name = name
             task.type = "regular"
-            self.data[task_name] = task
+            self.data[name] = task
 
-    def unregister(self, task_name):
+    def unregister(self, name):
         """Unregister task by name.
 
-        :param task_name: name of the task to unregister.
+        :param name: name of the task to unregister, or a
+        :class:`celery.task.Task` class with a valid ``name`` attribute.
 
         :raises NotRegistered: if the task has not been registered.
 
         """
-        if hasattr(task_name, "run"):
-            task_name = task_name.name
-        if task_name not in self.data:
+        if hasattr(name, "run"):
+            name = name.name
+        if name not in self.data:
             raise self.NotRegistered(
-                    "Task with name %s is not registered." % task_name)
-        del self.data[task_name]
+                    "Task with name %s is not registered." % name)
+        del self.data[name]
 
     def get_all(self):
         """Get all task types."""
@@ -86,9 +87,9 @@ class TaskRegistry(UserDict):
         """Get all periodic task types."""
         return self.filter_types(type="periodic")
 
-    def get_task(self, task_name):
+    def get_task(self, name):
         """Get task by name."""
-        return self.data[task_name]
+        return self.data[name]
 
 """
 .. data:: tasks
