@@ -68,7 +68,7 @@ class BaseAsyncResult(object):
 
         """
         status = self.backend.get_status(self.task_id)
-        return status != "PENDING" or status != "RETRY"
+        return status not in ["PENDING", "RETRY"]
 
     def successful(self):
         """Alias to :meth:`is_done`."""
@@ -266,7 +266,7 @@ class TaskSetResult(object):
         :returns: list of return values for all tasks in the taskset.
 
         """
-        timeout_timer = TimeoutTimer(timeout) # Timeout timer starts here.
+        timeout_timer = TimeoutTimer(timeout)
         results = PositionQueue(length=self.total)
 
         while True:
