@@ -2,7 +2,6 @@
 from carrot.connection import DjangoAMQPConnection
 from celery.messaging import TaskConsumer
 from celery.conf import DAEMON_CONCURRENCY, DAEMON_LOG_FILE
-from celery.conf import QUEUE_WAKEUP_AFTER, EMPTY_MSG_EMIT_EVERY
 from celery.conf import SEND_CELERY_TASK_ERROR_EMAILS
 from celery.log import setup_logger
 from celery.registry import tasks
@@ -13,7 +12,9 @@ from celery.timer import EventTimer
 from django.core.mail import mail_admins
 import multiprocessing
 import traceback
+import threading
 import logging
+import signal
 import socket
 import time
 import sys
@@ -29,16 +30,12 @@ The contents of the full traceback was:
 
 %%(traceback)s
 
-%%(EMAIL_SIGNATURE_SEP)s
+%(EMAIL_SIGNATURE_SEP)s
 Just thought I'd let you know!
 celeryd at %%(hostname)s.
 """ % {"EMAIL_SIGNATURE_SEP": EMAIL_SIGNATURE_SEP}
 
 
-class EmptyQueue(Exception):
-    """The message queue is currently empty."""
-
-
 class UnknownTask(Exception):
     """Got an unknown task in the queue. The message is requeued and
     ignored."""
@@ -71,11 +68,28 @@ def jail(task_id, func, args, kwargs):
     from django.db import connection
     connection.close()
 
-    # Convert any unicode keys in the keyword arguments to ascii.
-    kwargs = dict([(k.encode("utf-8"), v)
-                        for k, v in kwargs.items()])
+    # Reset cache connection only if using memcached/libmemcached
+    from django.core import cache
+    # XXX At Opera we use a custom memcached backend that uses libmemcached
+    # instead of libmemcache (cmemcache). Should find a better solution for
+    # this, but for now "memcached" should probably be unique enough of a
+    # string to not make problems.
+    cache_backend = cache.settings.CACHE_BACKEND
+    if hasattr(cache, "parse_backend_uri"):
+        cache_scheme = cache.parse_backend_uri(cache_backend)[0]
+    else:
+        # Django <= 1.0.2
+        cache_scheme = cache_backend.split(":", 1)[0]
+    if "memcached" in cache_scheme:
+        cache.cache.close()
+
+    # Backend process cleanup
+    default_backend.process_cleanup()
+
     try:
         result = func(*args, **kwargs)
+    except (SystemExit, KeyboardInterrupt):
+        raise
     except Exception, exc:
         default_backend.mark_as_failure(task_id, exc)
         return ExceptionInfo(sys.exc_info())
@@ -117,6 +131,10 @@ class TaskWrapper(object):
 
         Mapping of keyword arguments to apply to the task.
 
+    .. attribute:: message
+
+        The original message sent. Used for acknowledging the message.
+
     """
     success_msg = "Task %(name)s[%(id)s] processed: %(return_value)s"
     fail_msg = """
@@ -127,13 +145,15 @@ class TaskWrapper(object):
     """
     fail_email_body = TASK_FAIL_EMAIL_BODY
 
-    def __init__(self, task_name, task_id, task_func, args, kwargs, **opts):
+    def __init__(self, task_name, task_id, task_func, args, kwargs,
+            on_acknowledge=None, **opts):
         self.task_name = task_name
         self.task_id = task_id
         self.task_func = task_func
         self.args = args
         self.kwargs = kwargs
         self.logger = kwargs.get("logger")
+        self.on_acknowledge = on_acknowledge
         for opt in ("success_msg", "fail_msg", "fail_email_subject",
                 "fail_email_body"):
             setattr(self, opt, opts.get(opt, getattr(self, opt, None)))
@@ -147,7 +167,7 @@ class TaskWrapper(object):
                 self.args, self.kwargs)
 
     @classmethod
-    def from_message(cls, message, logger):
+    def from_message(cls, message, message_data, logger):
         """Create a :class:`TaskWrapper` from a task message sent by
         :class:`celery.messaging.TaskPublisher`.
 
@@ -157,15 +177,20 @@ class TaskWrapper(object):
         :returns: :class:`TaskWrapper` instance.
 
         """
-        message_data = message.decode()
         task_name = message_data["task"]
         task_id = message_data["id"]
         args = message_data["args"]
         kwargs = message_data["kwargs"]
+
+        # Convert any unicode keys in the keyword arguments to ascii.
+        kwargs = dict([(key.encode("utf-8"), value)
+                    for key, value in kwargs.items()])
+
         if task_name not in tasks:
             raise UnknownTask(task_name)
         task_func = tasks[task_name]
-        return cls(task_name, task_id, task_func, args, kwargs, logger=logger)
+        return cls(task_name, task_id, task_func, args, kwargs,
+                    on_acknowledge=message.ack, logger=logger)
 
     def extend_with_default_kwargs(self, loglevel, logfile):
         """Extend the tasks keyword arguments with standard task arguments.
@@ -190,6 +215,8 @@ class TaskWrapper(object):
 
         """
         task_func_kwargs = self.extend_with_default_kwargs(loglevel, logfile)
+        if self.on_acknowledge:
+            self.on_acknowledge()
         return jail(self.task_id, [
                         self.task_func, self.args, task_func_kwargs])
 
@@ -239,9 +266,41 @@ class TaskWrapper(object):
                      self.args, task_func_kwargs]
         return pool.apply_async(jail, args=jail_args,
                 callbacks=[self.on_success], errbacks=[self.on_failure],
+                on_acknowledge=self.on_acknowledge,
                 meta={"task_id": self.task_id, "task_name": self.task_name})
 
 
+class PeriodicWorkController(threading.Thread):
+    """A thread that continuously checks if there are
+    :class:`celery.task.PeriodicTask`s waiting for execution, and executes
+    them.
+
+    Example
+
+        >>> PeriodicWorkController().start()
+
+    """
+
+    def __init__(self):
+        super(PeriodicWorkController, self).__init__()
+        self._shutdown = threading.Event()
+        self._stopped = threading.Event()
+
+    def run(self):
+        """Don't use :meth:`run`. use :meth:`start`."""
+        while True:
+            if self._shutdown.isSet():
+                break
+            default_periodic_status_backend.run_periodic_tasks()
+            time.sleep(1)
+        self._stopped.set() # indicate that we are stopped
+
+    def stop(self):
+        """Shutdown the thread."""
+        self._shutdown.set()
+        self._stopped.wait() # block until this thread is done
+
+
 class WorkController(object):
     """Executes tasks waiting in the task queue.
 
@@ -251,8 +310,6 @@ class WorkController(object):
 
     :param loglevel: see :attr:`loglevel`.
 
-    :param queue_wakeup_after: see :attr:`queue_wakeup_after`.
-
 
     .. attribute:: concurrency
 
@@ -268,18 +325,6 @@ class WorkController(object):
         The logfile used, if no logfile is specified it uses ``stderr``
         (default: :const:`celery.conf.DAEMON_LOG_FILE`).
 
-    .. attribute:: queue_wakeup_after
-
-        The time it takes for the daemon to wake up after the queue is empty,
-        so it can check for more work
-        (default: :const:`celery.conf.QUEUE_WAKEUP_AFTER`).
-
-    .. attribute:: empty_msg_emit_every
-
-        How often the daemon emits the ``"Waiting for queue..."`` message.
-        If this is ``None``, the message will never be logged.
-        (default: :const:`celery.conf.EMPTY_MSG_EMIT_EVERY`)
-
     .. attribute:: logger
 
         The :class:`logging.Logger` instance used for logging.
@@ -296,22 +341,26 @@ class WorkController(object):
     loglevel = logging.ERROR
     concurrency = DAEMON_CONCURRENCY
     logfile = DAEMON_LOG_FILE
-    queue_wakeup_after = QUEUE_WAKEUP_AFTER
-    empty_msg_emit_every = EMPTY_MSG_EMIT_EVERY
+    _state = None
 
     def __init__(self, concurrency=None, logfile=None, loglevel=None,
-            queue_wakeup_after=None, is_detached=False):
+            is_detached=False):
         self.loglevel = loglevel or self.loglevel
         self.concurrency = concurrency or self.concurrency
         self.logfile = logfile or self.logfile
-        self.queue_wakeup_after = queue_wakeup_after or \
-                                    self.queue_wakeup_after
         self.logger = setup_logger(loglevel, logfile)
         self.pool = TaskPool(self.concurrency, logger=self.logger)
-        self.task_consumer = None
-        self.task_consumer_it = None
+        self.periodicworkcontroller = PeriodicWorkController()
         self.is_detached = is_detached
-        self.reset_connection()
+        self.amqp_connection = None
+        self.task_consumer = None
+
+    def close_connection(self):
+        """Close the AMQP connection."""
+        if self.task_consumer:
+            self.task_consumer.close()
+        if self.amqp_connection:
+            self.amqp_connection.close()
 
     def reset_connection(self):
         """Reset the AMQP connection, and reinitialize the
@@ -320,11 +369,11 @@ class WorkController(object):
         Resets the task consumer in :attr:`task_consumer`.
 
         """
-        if self.task_consumer:
-            self.task_consumer.connection.close()
-        amqp_connection = DjangoAMQPConnection()
-        self.task_consumer = TaskConsumer(connection=amqp_connection)
-        self.task_consumer_it = self.task_consumer.iterqueue(infinite=True)
+        self.close_connection()
+        self.amqp_connection = DjangoAMQPConnection()
+        self.task_consumer = TaskConsumer(connection=self.amqp_connection)
+        self.task_consumer.register_callback(self._message_callback)
+        return self.task_consumer
 
     def connection_diagnostics(self):
         """Diagnose the AMQP connection, and reset connection if
@@ -336,23 +385,27 @@ class WorkController(object):
                     "AMQP Connection has died, restoring connection.")
             self.reset_connection()
 
-    def receive_message(self):
-        """Receive the next message from the message broker.
+    def _message_callback(self, message_data, message):
+        """The method called when we receive a message."""
+        try:
+            try:
+                self.process_task(message_data, message)
+            except ValueError:
+                # execute_next_task didn't return a r/name/id tuple,
+                # probably because it got an exception.
+                pass
+            except UnknownTask, exc:
+                self.logger.info("Unknown task ignored: %s" % (exc))
+            except Exception, exc:
+                self.logger.critical("Message queue raised %s: %s\n%s" % (
+                                exc.__class__, exc, traceback.format_exc()))
+        except (SystemExit, KeyboardInterrupt):
+            self.shutdown()
 
-        Tries to reset the AMQP connection if not available.
-        Returns ``None`` if no message is waiting on the queue.
-
-        :rtype: :class:`carrot.messaging.Message` instance.
-
-        """
-        message = self.task_consumer_it.next()
-        if not message:
-            raise EmptyQueue()
-        return message
-
-    def process_task(self, message):
+    def process_task(self, message_data, message):
         """Process task message by passing it to the pool of workers."""
-        task = TaskWrapper.from_message(message, logger=self.logger)
+        task = TaskWrapper.from_message(message, message_data,
+                                        logger=self.logger)
         self.logger.info("Got task from broker: %s[%s]" % (
             task.task_name, task.task_id))
         self.logger.debug("Got a task: %s. Trying to execute it..." % task)
@@ -364,39 +417,24 @@ class WorkController(object):
 
         return result
 
-    def execute_next_task(self):
-        """Execute the next task on the queue using the multiprocessing pool.
-
-        Catches all exceptions and logs them with level
-        :const:`logging.CRITICAL`.
-
-        Raises :exc:`EmptyQueue` exception if there is no message
-        waiting on the queue.
-
-        """
-        self.process_task(self.receive_message())
-
-    def run_periodic_tasks(self):
-        """Schedule all waiting periodic tasks for execution.
-
-        """
-        self.logger.debug("Looking for periodic tasks ready for execution...")
-        default_periodic_status_backend.run_periodic_tasks()
-
-    def schedule_retry_tasks(self):
-        """Reschedule all requeued tasks waiting for retry."""
-        pass
+    def shutdown(self):
+        """Make sure ``celeryd`` exits cleanly."""
+        # shut down the periodic work controller thread
+        if self._state != "RUN":
+            return
+        self._state = "TERMINATE"
+        self.periodicworkcontroller.stop()
+        self.pool.terminate()
+        self.close_connection()
 
     def run(self):
         """Starts the workers main loop."""
-        log_wait = lambda: self.logger.info("Waiting for queue...")
-        ev_msg_waiting = EventTimer(log_wait, self.empty_msg_emit_every)
-        events = [
-            EventTimer(self.run_periodic_tasks, 1),
-            EventTimer(self.schedule_retry_tasks, 2),
-        ]
+        self._state = "RUN"
+        task_consumer = self.reset_connection()
+        it = task_consumer.iterconsume(limit=None)
 
         self.pool.run()
+        self.periodicworkcontroller.start()
 
         # If not running as daemon, and DEBUG logging level is enabled,
         # print pool PIDs and sleep for a second before we start.
@@ -406,22 +444,8 @@ class WorkController(object):
             if not self.is_detached:
                 time.sleep(1)
 
-        while True:
-            [event.tick() for event in events]
-            try:
-                self.execute_next_task()
-            except ValueError:
-                # execute_next_task didn't return a r/name/id tuple,
-                # probably because it got an exception.
-                continue
-            except EmptyQueue:
-                ev_msg_waiting.tick()
-                time.sleep(self.queue_wakeup_after)
-                continue
-            except UnknownTask, exc:
-                self.logger.info("Unknown task ignored: %s" % (exc))
-                continue
-            except Exception, exc:
-                self.logger.critical("Message queue raised %s: %s\n%s" % (
-                             exc.__class__, exc, traceback.format_exc()))
-                continue
+        try:
+            while True:
+                it.next()
+        except (SystemExit, KeyboardInterrupt):
+            self.shutdown()
